/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file RPCConsole.h
 *    Console that allows to launch RPC calls easily
 *
 * @author Erik Einhorn
 * @date   2012/09/13
 */

// include the view base class
#include <rcp/ViewPart.h>
#include <serialization/GetterSetter.h>

namespace mira { 

///////////////////////////////////////////////////////////////////////////////

/**
 * Console that allows to launch RPC calls easily
 */
class RPCConsole : public mira::ViewPart
{
Q_OBJECT
MIRA_META_OBJECT(RPCConsole,
                ("Name", "RPC Console")
                ("Description", "Console with auto-completion that allows to launch RPC calls easily"))

public:
	RPCConsole();

	template<typename Reflector>
	void reflect(Reflector& r)
	{
		mira::ViewPart::reflect(r);
		r.member("History", getter(&RPCConsole::getHistory,this),
		                    setter(&RPCConsole::setHistory,this),
		                    "The command history", std::list<std::string>());
	}

	virtual Qt::DockWidgetArea getDefaultDockArea() const {
		return Qt::BottomDockWidgetArea;
	}

protected:

    virtual QWidget* createPartControl();

private slots:

	// add slots

private:

	const std::list<std::string>& getHistory();
	void setHistory(const std::list<std::string>& history);

private:

	class Console;
    Console* mConsole;
    std::list<std::string> mHistory;

};

///////////////////////////////////////////////////////////////////////////////

}
