/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file MIRALaunchGUI.C
 *    Description.
 *
 * @author Erik Einhorn
 * @date   2010/11
 */

#include <QApplication>

#include <fw/FrameworkWithGui.h>

using namespace std;
using namespace mira;

int main(int argc, char** argv)
{
	int res = -1;
	try {
		FrameworkWithGui fw(argc, argv);
		ProgramOptions::VariableMap vmap = MIRA_CMDLINE.getOptions();
		if ( vmap.count("help") ) {
			std::cout << "Usage:" << std::endl << std::endl;
			std::cout << "    miragui [options] [config file1] [config file2] ... [config fileN]" << std::endl << std::endl;
			std::cout << " Starts a MIRA process with Qt GUI support. Started units can show Qt widgets, etc." <<  std::endl;
			std::cout << " Each configuration file can be specified as follows:" << std::endl;
			std::cout << "    <path>           : Relative or absolute path (mypath/myconfig.xml)" << std::endl;
			std::cout << "    <package>:<path> : Path relative to a package (mypackage:etc/myconfig.xml)" << std::endl;
			std::cout << "    ${find <path>}   : Searches for the specified file in all mira paths recursively\n"
					     "                                                       (\"\\${find somefile.xml}\")" << std::endl<< std::endl;
			std::cout << " Options:" << std::endl << std::endl;
			std::cout << MIRA_CMDLINE.getDescriptions();
			return 0;
		}

		res = fw.run();
	} catch(std::exception& ex) {
		MIRA_LOG_EXCEPTION(ERROR, ex) << "An exception has occured:\n";
		return -1;
	}

	MIRA_LOG(NOTICE) << "Exiting application";
	return res;
}
